@@ -16,6 +16,7 @@ import pykat.external.six as six
 if six.PY2:
 	import exceptions
 
+import math
 import warnings
 import pykat.exceptions as pkex
 import pykat
@@ -24,6 +25,7 @@ from pykat.exceptions import *
 import abc
 import copy
 from collections import OrderedDict
+from .optics import ABCD
 
 if HAS_OPTIVIS:
     import optivis.bench.components as optivis_components
@@ -85,6 +87,11 @@ class NodeGaussSetter(object):
     @qy.setter
     def qy(self, value):
         self.__node().setGauss(self.__comp(), self.qx, complex(value))
+     
+    @property 
+    def n(self):
+        return self.__node().n
+    
         
 id_____pykat_class = 0
  
@@ -133,6 +140,7 @@ class Component(object):
         # of this one, otherwise we're making a copy of a copy of a copy...
         result = self.__class__.__new__(self.__class__.__base__)
         result._unfreeze()
+        memo[id(self)] = result
         result.__dict__ = copy.deepcopy(self.__dict__, memo)
         
         for _ in result._params:
@@ -171,7 +179,10 @@ class Component(object):
         self._kat = kat
         
         kat.nodes.registerComponentNodes(self, self._requested_node_names, self.__on_node_change)
-     
+    
+    def __repr__(self):
+        return "<%s (%s) at %s>" % (self.__class__.__name__, self.__name, hex(id(self)))
+        
     def _on_kat_remove(self):
         # inform all parameters that we have removed its owner
         # so that it can then warn about any puts/vars/xaxis
@@ -244,6 +255,17 @@ class Component(object):
             
         return None      
     
+    def ABCD(self, from_node, to_node, direction="x"):
+        """
+        Returns a 2x2 complex ABCD matrix for this component.
+        Actual matrix depends from which to which node you go.
+        
+        from_node: node object or node name as string
+        to_node:   node object or node name as string
+        direction: 'x' horizontal beam shape, 'y' vertical beam shape
+        """
+        return np.eye(2)
+    
     @property
     def removed(self): return self.__removed
     
@@ -301,16 +323,21 @@ class AbstractMirrorComponent(Component):
     def __init__(self, name, R=None, T=None, L=None, phi=0, Rcx=None, Rcy=None, xbeta=None, ybeta=None, mass=None, r_ap=None, Ix=None, Iy=None, zmech=None, rxmech=None, rymech=None):
         super(AbstractMirrorComponent, self).__init__(name)
  
-        if (L != None and R != None and T != None) and SIfloat(R)+SIfloat(T)+SIfloat(L) != 1: 
-            raise pkex.BasePyKatException('L+R+T must equal 1 if all are specified at {0}'.format(self.name))
-        elif (R != None and L is None and T != None):
-            L = 1- (SIfloat(R)+SIfloat(T))
-        elif (R is None and L != None and T != None):
-            R = 1 - (SIfloat(L)+SIfloat(T))
-        elif (R != None and L != None and T is None):
-            T = 1 - (SIfloat(L)+SIfloat(R))
-        elif (L is None and R is None and T is None):
-            raise pkex.BasePyKatException('Must specify at least two of L, R or T')
+        # This checking is no longer done on creating the object. Pykat will only complain about 
+        # R,T,L values when generating the actual finesse command. Up to that point you can set what
+        # ever you like. This is because we can now set parameters to be `constants` values, which 
+        # can be changed and are only final when we write it to the a finesse file.
+        
+        # if (L != None and R != None and T != None) and SIfloat(R)+SIfloat(T)+SIfloat(L) != 1:
+        #     raise pkex.BasePyKatException('L+R+T must equal 1 if all are specified at {0}'.format(self.name))
+        # elif (R != None and L is None and T != None):
+        #     L = 1- (SIfloat(R)+SIfloat(T))
+        # elif (R is None and L != None and T != None):
+        #     R = 1 - (SIfloat(L)+SIfloat(T))
+        # elif (R != None and L != None and T is None):
+        #     T = 1 - (SIfloat(L)+SIfloat(R))
+        # elif (L is None and R is None and T is None):
+        #     raise pkex.BasePyKatException('Must specify at least two of L, R or T')
         
         self.__R = Param("R", self, SIfloat(R))
         self.__T = Param("T", self, SIfloat(T))
@@ -319,8 +346,8 @@ class AbstractMirrorComponent(Component):
         self.__phi = Param("phi", self, SIfloat(phi), canFsig=True, fsig_name="phase")
         self.__Rcx = AttrParam("Rcx", self, SIfloat(Rcx))
         self.__Rcy = AttrParam("Rcy", self, SIfloat(Rcy))
-        self.__xbeta = AttrParam("xbeta", self, SIfloat(xbeta), canFsig=True, fsig_name="xbeta")
-        self.__ybeta = AttrParam("ybeta", self, SIfloat(ybeta), canFsig=True, fsig_name="ybeta")
+        self.__xbeta = AttrParam("xbeta", self, SIfloat(xbeta), canFsig=True, fsig_name="xbeta", fsig_name_options=["yaw", "rx"])
+        self.__ybeta = AttrParam("ybeta", self, SIfloat(ybeta), canFsig=True, fsig_name="ybeta", fsig_name_options=["pitch", "ry"])
         self.__mass = AttrParam("mass", self, SIfloat(mass))
         self.__Ix = AttrParam("Ix", self, SIfloat(Ix))
         self.__Iy = AttrParam("Iy", self, SIfloat(Iy))
@@ -345,11 +372,21 @@ class AbstractMirrorComponent(Component):
         self._default_fsig_param = self.__phi
     
     def setRTL(self, R=None, T=None, L=None):
-        if R is not None: self.R.value = R
-        if T is not None: self.T.value = T
-        if L is not None: self.L.value = L
+        """
+        Sets the R, T, and/or L properties. At least two values should be specfified
+        as the the third can be assumed as:
+        
+            R+T+L = 1
+        
+        By setting one of the properties to None will set whether an m/m1/m2 (bs/bs1/bs2)
+        command is used with Finesse.
+        """
+        if R is not None: self.R = R
+        if T is not None: self.T = T
+        if L is not None: self.L = L
 
     def completeRTL(self, R=None, T=None, L=None):
+        
         setValues = sum(x is not None for x in [R,T,L])
         if setValues == 3:
             self.setRTL(R,T,L)
@@ -357,17 +394,17 @@ class AbstractMirrorComponent(Component):
             raise pkex.BasePyKatException("must set at least two out of three parameters (R, T, L)")            
         else:
             if R is not None:
-                self.R.value = R
+                self.R = R
             else:
-                self.R.value = 1-T-L            
+                self.R = 1-T-L            
             if T is not None:
-                self.T.value = T
+                self.T = T
             else:
-                self.T.value = 1-R-L            
+                self.T = 1-R-L            
             if L is not None:
-                self.L.value = L
+                self.L = L
             else:
-                self.L.value = 1-R-T            
+                self.L = 1-R-T            
 
     @property
     def z(self): return self.__z
@@ -540,14 +577,44 @@ class mirror(AbstractMirrorComponent):
                 return mirror(values[0], values[4], values[5], T=None, R=values[1], L=values[2], phi=values[3])
 
     def getFinesseText(self):
-        if abs(self.R + self.T + self.L - 1) > 1e-14:
-            raise pkex.BasePyKatException("Mirror {0} has R+T+L = {1}, must equal 1 +- 1e-14".format(self.name, self.R+self.T+self.L))
+        mode = 'm'
+        opt1 = None
+        opt2 = None
         
+        if self.R.value is not None and self.T.value is not None and self.L.value is None:
+            mode = 'm'
+            opt1 = self.R
+            opt2 = self.T
+            msg = "R+T"
+        elif self.R.value is None and self.T.value is not None and self.L.value is not None:
+            mode = 'm1'
+            opt1 = self.T
+            opt2 = self.L
+            msg = "T+L"
+        elif self.R.value is not None and self.T.value is None and self.L.value is not None:
+            mode = 'm2'
+            opt1 = self.R
+            opt2 = self.L
+            msg = "R+L"
+        elif self.R.value is not None and self.T.value is not None and self.L.value is not None:
+            mode = 'm'
+            opt1 = self.R
+            opt2 = self.T
+            v = opt1.value + opt2.value + self.L.value
+            
+            if v > 1+1e-14 or v < 0:
+                raise pkex.BasePyKatException("Mirror {0} has R+T+L = {1}, must be, 0 < R+T+L <= 1".format(self.name, v))
+        else:
+            raise pkex.BasePyKatException("Mirror {0} has R={1}, T={2} and L={3}. You must define at least a pair of values".format(self.name, self.R, self.T, self.L))
+        
+        if opt1.value + opt2.value > 1+1e-14:
+            raise pkex.BasePyKatException("Mirror {0} has {1} = {2}, must be <= 1".format(self.name, msg, opt1+opt2))
+            
         rtn = []
             
-        rtn.append('m {0} {1} {2} {3} {4} {5}'.format(
-                self.name, self.R.value, self.T.value, self.phi.value,
-                self.nodes[0].name, self.nodes[1].name))
+        rtn.append('{mode} {0} {1} {2} {3} {4} {5}'.format(
+                self.name, opt1, opt2, self.phi,
+                self.nodes[0].name, self.nodes[1].name, mode=mode))
 
         for p in self._params:
             rtn.extend(p.getFinesseText())
@@ -593,6 +660,37 @@ class mirror(AbstractMirrorComponent):
             self._svgItem = pykat.gui.graphics.ComponentQGraphicsItem(":/resources/mirror_flat.svg", self ,[(-4,15,self.nodes[0]), (14,15,self.nodes[1])])
             
         return self._svgItem
+        
+    def ABCD(self, from_node, to_node, direction="x"):
+        direction = direction.lower()
+        from_node = str(from_node)
+        to_node = str(to_node)
+        
+        if direction == "x" and self.Rcx.value is not None:
+            Rc = self.Rcx.value
+        elif direction == "y" and self.Rcy.value is not None:
+            Rc = self.Rcy.value
+        else:
+            Rc = math.inf
+        
+        n1 = self.nodes[0].n
+        n2 = self.nodes[1].n
+        
+        if from_node == str(self.nodes[0]) and to_node == str(self.nodes[0]):
+            return ABCD.mirror_refl(n1, Rc)
+        elif from_node == str(self.nodes[0]) and to_node == str(self.nodes[1]):
+            return ABCD.mirror_trans(n1, n2, Rc)
+        elif from_node == str(self.nodes[1]) and to_node == str(self.nodes[0]):
+            return ABCD.mirror_tran(n2, n1, -Rc)
+        elif from_node == str(self.nodes[1]) and to_node == str(self.nodes[1]):
+            return ABCD.mirror_refl(n2, -Rc)
+        else:
+            raise pkex.BasePyKatException("Check nodes {} and {} are nodes of {}".format(from_node, to_node, self.name))
+    
+    def nodeConnections(self):
+        return (
+                   (self.nodes[0], self.nodes[1]),
+               )
 
 class beamSplitter(AbstractMirrorComponent):
     def __init__(self, name, node1, node2, node3, node4, R = None, T = None, L=None, phi = 0, alpha = 0, Rcx = None, Rcy = None, xbeta = None, ybeta = None, mass = None, r_ap = None):
@@ -677,16 +775,46 @@ class beamSplitter(AbstractMirrorComponent):
                                 values[1], None, values[2], values[3], values[4])
         
     def getFinesseText(self):
-        if abs(self.R + self.T + self.L - 1) > 1e-14:
-            raise pkex.BasePyKatException("Beamsplitter {0} has R+T+L = {1}, must equal 1 +- 1e-14".format(self.name, self.R+self.T+self.L))
-
+        mode = 'bs'
+        opt1 = None
+        opt2 = None
+        
+        if self.R.value is not None and self.T.value is not None and self.L.value is None:
+            mode = 'bs'
+            opt1 = self.R
+            opt2 = self.T
+            msg = "R+T"
+        elif self.R.value is None and self.T.value is not None and self.L.value is not None:
+            mode = 'bs1'
+            opt1 = self.T
+            opt2 = self.L
+            msg = "T+L"
+        elif self.R.value is not None and self.T.value is None and self.L.value is not None:
+            mode = 'bs2'
+            opt1 = self.R
+            opt2 = self.L
+            msg = "R+L"
+        elif self.R.value is not None and self.T.value is not None and self.L.value is not None:
+            mode = 'bs'
+            opt1 = self.R
+            opt2 = self.T
+            v = opt1.value + opt2.value + self.L.value
+            
+            if v > 1+1e-14 or v < 0:
+                raise pkex.BasePyKatException("Beamsplitter {0} has R+T+L = {1}, must be, 0 < R+T+L <= 1".format(self.name, v))
+        else:
+            raise pkex.BasePyKatException("Beamsplitter {0} has R={1}, T={2} and L={3}. You must define at least a pair of values".format(self.name, self.R.value,self.T.value,self.L.value))
+        
+        if opt1.value + opt2.value > 1+1e-14:
+            raise pkex.BasePyKatException("Beamsplitter {0} has {1} = {2}, must be <= 1".format(self.name, msg, opt1+opt2))
+            
         rtn = []
             
-        rtn.append('bs {0} {1} {2} {3} {4} {5} {6} {7} {8}'.format(
-                self.name, self.R.value, self.T.value, self.phi.value,
-                self.alpha.value, self.nodes[0].name,
+        rtn.append('{mode} {0} {1} {2} {3} {4} {5} {6} {7} {8}'.format(
+                self.name, opt1, opt2, self.phi,
+                self.alpha, self.nodes[0].name,
                 self.nodes[1].name, self.nodes[2].name,
-                self.nodes[3].name))
+                self.nodes[3].name, mode=mode))
 
         for p in self._params:
             rtn.extend(p.getFinesseText())
@@ -702,9 +830,56 @@ class beamSplitter(AbstractMirrorComponent):
             self._svgItem = pykat.gui.graphics.ComponentQGraphicsItem(":/resources/mirror_flat.svg", self ,[(-4,24,self.nodes[0]), (-4,6,self.nodes[1]), (14,6,self.nodes[2]), (14,24,self.nodes[3])])
             
         return self._svgItem
+        
+    def ABCD(self, from_node, to_node, direction="x"):
+        direction = direction.lower()
+        from_node = str(from_node)
+        to_node = str(to_node)
    
+        if direction == "x":
+            refl = ABCD.bs_refl_x
+            trans = ABCD.bs_trans_x
+        else:
+            refl = ABCD.bs_refl_y
+            trans = ABCD.bs_trans_y
+       
+        if direction == "x" and self.Rcx.value is not None:
+            Rc = self.Rcx.value
+        elif direction == "y" and self.Rcy.value is not None:
+            Rc = self.Rcy.value
+        else:
+            Rc = math.inf
+   
+        n1 = self.nodes[0].n
+        n2 = self.nodes[3].n
+   
+        if (from_node == str(self.nodes[0]) and to_node == str(self.nodes[1])) or \
+            (from_node == str(self.nodes[1]) and to_node == str(self.nodes[0])):
+            return refl(n1, Rc, self.alpha.value)
+        elif (from_node == str(self.nodes[2]) and to_node == str(self.nodes[3])) or \
+            (from_node == str(self.nodes[3]) and to_node == str(self.nodes[2])):
+            return refl(n2, -Rc, self.alpha.value)
+        elif from_node == str(self.nodes[0]) and to_node == str(self.nodes[2]):
+            return trans(n1, n2, Rc, self.alpha.value)
+        elif from_node == str(self.nodes[2]) and to_node == str(self.nodes[0]):
+            return trans(n2, n1, -Rc, self.alpha.value)
+        elif from_node == str(self.nodes[1]) and to_node == str(self.nodes[3]):
+            return trans(n1, n2, Rc, self.alpha.value)
+        elif from_node == str(self.nodes[3]) and to_node == str(self.nodes[1]):
+            return trans(n2, n1, -Rc, self.alpha.value)
+        else:
+            raise pkex.BasePyKatException("Check node combination {} and {} for {}".format(from_node, to_node, self.name))
+            
+    def nodeConnections(self):
+        return (
+                   (self.nodes[0], self.nodes[1]), # node 1 <-R> node 2
+                   (self.nodes[0], self.nodes[2]), # node 1 <-T> node 3
+                   (self.nodes[1], self.nodes[3]), # node 2 <-T> node 4
+                   (self.nodes[2], self.nodes[3]), # node 3 <-R> node 4
+               )
+        
 class space(Component):
-    def __init__(self, name, node1, node2, L = 0, n = 1, g = None, gx = None, gy = None):
+    def __init__(self, name, node1, node2, L = 0, n = 1, gx = None, gy = None):
         Component.__init__(self, name)
         
         self._requested_node_names.append(node1)
@@ -730,22 +905,32 @@ class space(Component):
     def n(self,value): self.__n.value = SIfloat(value)
 
     @property
-    def g(self):
+    def gouy(self):
         if self.__gx.value == self.__gy.value: 
-            return self.__g
+            return self.__gx.value 
         else:
             raise pkex.BasePyKatException("Gouy phase in x and y directions are different, use gx and gy properties instead")
             
-    @g.setter
-    def g(self,value):
+    @gouy.setter
+    def gouy(self,value):
         self.__gx.value = SIfloat(value)
         self.__gy.value = SIfloat(value)
         
+    @property
+    def gouyx(self): return self.gx
+    @gouyx.setter
+    def gouyx(self,value): self.gx = SIfloat(value)
+
+    @property
+    def gouyy(self): return self.gy
+    @gouyy.setter
+    def gouyy(self,value): self.gy = SIfloat(value)
+
     @property
     def gx(self): return self.__gx
     @gx.setter
     def gx(self,value): self.__gx.value = SIfloat(value)
-
+    
     @property
     def gy(self): return self.__gy
     @gy.setter
@@ -796,9 +981,9 @@ class space(Component):
         rtn = []
         
         if self.__n == 1:
-            rtn.append('s {0} {1} {2} {3}'.format(self.name, self.__L.value, self.nodes[0].name, self.nodes[1].name))
+            rtn.append('s {0} {1} {2} {3}'.format(self.name, self.__L, self.nodes[0].name, self.nodes[1].name))
         else:
-            rtn.append('s {0} {1} {2} {3} {4}'.format(self.name, self.__L.value, self.__n.value, self.nodes[0].name, self.nodes[1].name))
+            rtn.append('s {0} {1} {2} {3} {4}'.format(self.name, self.__L, self.__n, self.nodes[0].name, self.nodes[1].name))
        
         for p in self._params:
             rtn.extend(p.getFinesseText())
@@ -814,6 +999,14 @@ class space(Component):
         
         return self._QItem
 
+    def ABCD(self, from_node, to_node, direction="x"):
+        return ABCD.space(self.n.value, self.L.value)
+        
+    def nodeConnections(self):
+        return (
+                   (self.nodes[0], self.nodes[1]),
+               )
+               
 class grating(Component):
     def __init__(self, name, node1, node2, node3 = None, node4 = None, n = 2, d = 0, eta_0 = None, eta_1 = None, eta_2 = None, eta_3 = None, rho_0 = None, alpha = None): # TODO: implement Rcx, Rcy and Rc
         Component.__init__(self, name)
@@ -929,11 +1122,11 @@ class grating(Component):
         rtn = []
         
         if self.__n == 2:
-            rtn.append('gr{0} {1} {2} {3} {4}'.format(self.__n, self.name, self.__d.value, self.nodes[0].name, self.nodes[1].name))
+            rtn.append('gr{0} {1} {2} {3} {4}'.format(self.__n, self.name, self.__d, self.nodes[0].name, self.nodes[1].name))
         elif self.__n == 3:
-            rtn.append('gr{0} {1} {2} {3} {4} {5}'.format(self.__n, self.name, self.__d.value, self.nodes[0].name, self.nodes[1].name, self.nodes[2].name))
+            rtn.append('gr{0} {1} {2} {3} {4} {5}'.format(self.__n, self.name, self.__d, self.nodes[0].name, self.nodes[1].name, self.nodes[2].name))
         else:
-            rtn.append('gr{0} {1} {2} {3} {4} {5} {6}'.format(self.__n, self.name, self.__d.value, self.nodes[0].name, self.nodes[1].name, self.nodes[2].name, self.nodes[3].name))
+            rtn.append('gr{0} {1} {2} {3} {4} {5} {6}'.format(self.__n, self.name, self.__d, self.nodes[0].name, self.nodes[1].name, self.nodes[2].name, self.nodes[3].name))
         
         for p in self._params:
             rtn.extend(p.getFinesseText())
@@ -950,31 +1143,20 @@ class grating(Component):
         return self._svgItem
 
 class isolator(Component):
-    def __init__(self, name, node1, node2, S = 0, node3="dump", option = 0, L = 0):
+    def __init__(self, name, node1, node2, S):
         """
-        Creates an isolator component. Ligth passes from node 1 to node 2, and from node
-        2 to node 3. 
+        Creates an isolator component. Ligth passes from node 1 to node 2.
 
-        S         Suppression factor for the reversed direction [power dB].
-        L         Loss, fraction of input power loss. Number between 0 and 1.
-        option    0: Light passes from node1 to node2, and from node2 to node3. Light
-                     is suppressed when going from node3 to node2, and from node2 to
-                     node1.
-                  1: Light passes from node2 to node1, and from node3 to node2. Light
-                     is suppressed when going from node1 to node2, and from node2 to
-                     node3. 
+        S         Suppression factor for the reversed direction [power dB]
         """
         
         Component.__init__(self, name)
         
         self._requested_node_names.append(node1)
         self._requested_node_names.append(node2)
-        self._requested_node_names.append(node3)
         self._svgItem = None
-        self._option = option
         
         self.__S = Param("S",self,SIfloat(S))
-        self.__L = Param("L",self,SIfloat(L))
 
         self._freeze()
         
@@ -982,49 +1164,23 @@ class isolator(Component):
     def S(self): return self.__S
     @S.setter
     def S(self, value): self.__S.value = SIfloat(value)
-
-    @property
-    def L(self): return self.__L
-    @L.setter
-    def L(self, value): self.__L.value = SIfloat(value)
     
     @staticmethod
     def parseFinesseText(text):
         values = text.split()
 
-        if values[0] != "isol" and values[0] != "isol*":
+        if values[0] != "isol":
             raise pkex.BasePyKatException("'{0}' not a valid Finesse isolator command".format(text))
-
-        if values[0].endswith('*'):
-            option = 1
-        else:
-            option = 0
             
         values.pop(0) # remove initial value
         
         if len(values) == 4:
-            return isolator(values[0], values[2], values[3], values[1], option=option)
-        elif len(values) == 5:
-            # Checking if loss is specified, should be a number.
-            if values[2].isnumeric():
-                return isolator(values[0], values[3], values[4], values[1],
-                                L=values[2], option=option)
-            # .. if not a number, it's a node name.
-            else:
-                return isolator(values[0], values[2], values[3], node3=values[4],
-                                S=values[1], option=option)
-        elif len(values) == 6:
-             return isolator(values[0], values[3], values[4], values[1], values[5], option, values[2])
+            return isolator(values[0], values[2], values[3], values[1])
         else:
             raise pkex.BasePyKatException("Isolator Finesse code format incorrect '{0}'".format(text))
         
     def getFinesseText(self):
-        if self._option == 0:
-            cmd = "isol"
-        elif self._option == 1:
-            cmd = "isol*"
-            
-        rtn = ['{cmd} {0} {1} {2} {3} {4} {5}'.format(self.name, self.S.value, self.L.value, self.nodes[0].name, self.nodes[1].name, self.nodes[2].name, cmd=cmd)]
+        rtn = ['isol {0} {1} {2} {3} {4} {5}'.format(self.name, self.S, self.nodes[0].name, self.nodes[1].name)]
         
         for p in self._params:
             rtn.extend(p.getFinesseText())
@@ -1066,12 +1222,26 @@ class isolator(Component):
         
         return self._svgItem
         
+    def nodeConnections(self):
+        return (
+                   (self.nodes[0], self.nodes[1])
+               )
         
-        
-class isolator1(Component):
+class dbs(Component):
     def __init__(self, name, node1, node2, node3, node4):
         """
-        Creates a 4-port isolator component.
+        Creates a 4-port isolator (directional beamsplitter) component.
+        
+        An unphysical component that separates beams depending on the direction:
+            node1 -> node3
+            node3 -> node4
+            node4 -> node2
+            node2 -> node1
+        
+        This component acts like a perfect isolator, and is often used in such
+        cases. Can be used to inject squeezing or as the input faraday isolator.
+        Losses can be applied by putting lossy mirrors in each of the beam paths
+        as required.
         """
         
         Component.__init__(self, name)
@@ -1088,18 +1258,18 @@ class isolator1(Component):
     def parseFinesseText(text):
         values = text.split()
 
-        if values[0] != "isol1":
+        if values[0] != "dbs":
             raise pkex.BasePyKatException("'{0}' not a valid Finesse isolator command".format(text))
             
         values.pop(0) # remove initial value
         
         if len(values) == 5:
-             return isolator1(values[0], values[1], values[2], values[3], values[4])
+             return dbs(values[0], values[1], values[2], values[3], values[4])
         else:
-            raise pkex.BasePyKatException("Isolator1 Finesse code format incorrect '{0}'".format(text))
+            raise pkex.BasePyKatException("dbs Finesse code format incorrect '{0}'".format(text))
         
     def getFinesseText(self):
-        rtn = ['isol1 {0} {1} {2} {3} {4}'.format(self.name, self.nodes[0].name,
+        rtn = ['dbs {0} {1} {2} {3} {4}'.format(self.name, self.nodes[0].name,
                                               self.nodes[1].name, self.nodes[2].name,
                                               self.nodes[3].name)]
         
@@ -1111,7 +1281,14 @@ class isolator1(Component):
     def getQGraphicsItem(self):
         raise NotImplemented()    
         
-
+    def nodeConnections(self):
+        return (
+                   (self.nodes[0], self.nodes[2]),
+                   (self.nodes[1], self.nodes[0]),
+                   (self.nodes[2], self.nodes[3]),
+                   (self.nodes[3], self.nodes[1]),
+               )
+               
 class lens(Component):
     def __init__(self, name, node1, node2, f=1, p=None):
         Component.__init__(self, name)
@@ -1164,9 +1341,9 @@ class lens(Component):
         
     def getFinesseText(self):
         if self.__p.value is None:
-            rtn = ['lens {0} {1} {2} {3}'.format(self.name, self.f.value, self.nodes[0].name, self.nodes[1].name)]
+            rtn = ['lens {0} {1} {2} {3}'.format(self.name, self.f, self.nodes[0].name, self.nodes[1].name)]
         else:
-            rtn = ['lens* {0} {1} {2} {3}'.format(self.name, self.p.value, self.nodes[0].name, self.nodes[1].name)]
+            rtn = ['lens* {0} {1} {2} {3}'.format(self.name, self.p, self.nodes[0].name, self.nodes[1].name)]
         
         for p in self._params:
             rtn.extend(p.getFinesseText())
@@ -1207,6 +1384,17 @@ class lens(Component):
         
         return self._svgItem
         
+    def ABCD(self, from_node, to_node, direction="x"):
+        if self.f.value is not None:
+            return ABCD.lens(self.f.value)
+        else:
+            return ABCD.lens(1/self.p.value)
+    
+    def nodeConnections(self):
+        return (
+                   (self.nodes[0], self.nodes[1]),
+               )
+                   
 class modulator(Component):
     def __init__(self, name, node1, node2, f, midx, order, modulation_type='pm', phase=0):
         Component.__init__(self, name)
@@ -1326,7 +1514,12 @@ class modulator(Component):
             self._svgItem = pykat.gui.graphics.ComponentQGraphicsItem(":/resources/modulator.svg", self ,[(-4,15,self.nodes[0]), (14,15,self.nodes[1])])
         
         return self._svgItem
-
+    
+    def nodeConnections(self):
+        return (
+                   (self.nodes[0], self.nodes[1]),
+               )
+               
 class laser(Component):
     def __init__(self, name, node, P=1, f=0, phase=0):
         Component.__init__(self,name)
@@ -1334,11 +1527,14 @@ class laser(Component):
         self._requested_node_names.append(node)
         
         self.__power = Param("P", self, SIfloat(P), canFsig=True, fsig_name="amp")
-        self.__f_offset = Param("f", self, f, canFsig=True, fsig_name="freq")
+        self.__f_offset = Param("f", self, None, canFsig=True, fsig_name="freq")
         self.__phase = Param("phase", self, SIfloat(phase), canFsig=True, fsig_name="phase")
+        self.__w0 = Param("phase", self, None, canFsig=True, fsig_name="w0", isPutable=False, isPutter=False, isTunable=False)
+        self.__z  = Param("phase", self, None, canFsig=True, fsig_name="z", isPutable=False, isPutter=False, isTunable=False)
         self.__noise = AttrParam("noise", self, None)
         self._svgItem = None
         
+        self.f = f
         self._default_fsig_param = self.__f_offset
 
         self._freeze()
@@ -1387,7 +1583,7 @@ class laser(Component):
             raise exceptions.FinesseParse("Laser Finesse code format incorrect '{0}'".format(text))
     
     def getFinesseText(self):
-        rtn = ['l {0} {1} {2} {3} {4}'.format(self.name, self.__power.value, self.__f_offset.value, self.__phase.value, self.nodes[0].name)]
+        rtn = ['l {0} {1} {2} {3} {4}'.format(self.name, self.__power, self.__f_offset, self.__phase, self.nodes[0].name)]
         
         for p in self._params:
             rtn.extend(p.getFinesseText())
@@ -1424,7 +1620,10 @@ class laser(Component):
             self._svgItem = pykat.gui.graphics.ComponentQGraphicsItem(":/resources/laser.svg", self, [(65,25,self.nodes[0])])
             
         return self._svgItem
-
+    
+    def nodeConnections(self):
+        return tuple()
+        
 class squeezer(Component):
     def __init__(self, name, node, f=0, db=0, angle=0, phase=0, entangled_carrier=False):
         Component.__init__(self,name)
@@ -1490,9 +1689,9 @@ class squeezer(Component):
     
     def getFinesseText(self):
         if self.entangled_carrier:
-            rtn = ['sq* {0} {1} {2} {3} {4}'.format(self.name, self.f.value, self.db.value, self.angle.value, self.nodes[0].name)]
+            rtn = ['sq* {0} {1} {2} {3} {4}'.format(self.name, self.f, self.db, self.angle, self.nodes[0].name)]
         else:
-            rtn = ['sq {0} {1} {2} {3} {4}'.format(self.name, self.f.value, self.db.value, self.angle.value, self.nodes[0].name)]
+            rtn = ['sq {0} {1} {2} {3} {4}'.format(self.name, self.f, self.db, self.angle, self.nodes[0].name)]
         
         for p in self._params:
             rtn.extend(p.getFinesseText())
@@ -1508,3 +1707,5 @@ class squeezer(Component):
             
         return self._svgItem
             
+    def nodeConnections(self):
+        return tuple()
\ No newline at end of file
