@@ -112,6 +112,7 @@ class BaseDetector(object) :
         # of this one, otherwise we're making a copy of a copy of a copy...
         result = self.__class__.__new__(self.__class__.__base__)
         result._unfreeze()
+        memo[id(self)] = result
         result.__dict__ = copy.deepcopy(self.__dict__, memo)
         
         result._freeze()
@@ -182,7 +183,7 @@ class BaseDetector(object) :
                 
         self._mask[_id] = factor
 
-    def _set_node(value, index):
+    def _set_node(self, value, index):
         if self._kat is None:
             raise pkex.BasePyKatException("This detector has not been added to a kat object yet")
         else:
@@ -223,7 +224,6 @@ class Detector1(BaseDetector):
     def node(self, value):
         self._set_node(value, 0)      
         
-                
 class Detector2(BaseDetector):
     """
     A detector that attaches to two node.
@@ -285,7 +285,7 @@ class beam(Detector1):
         if self.f.value is None:
             rtn.append("beam {name} {node}{alt}".format(name=self.name, node=self.node.name, alt=alt))
         else:
-            rtn.append("beam {name} {f} {node}{alt}".format(name=self.name, f=str(self.f.value), node=self.node.name, alt=alt))
+            rtn.append("beam {name} {f} {node}{alt}".format(name=self.name, f=self.f, node=self.node.name, alt=alt))
         
         for p in self._params:
             rtn.extend(p.getFinesseText())
@@ -383,11 +383,24 @@ class ad(Detector1):
     def __init__(self, name, frequency, node_name, mode=None, alternate_beam=False):
         BaseDetector.__init__(self, name, node_name)
         self.mode = mode
+        
         self.alternate_beam = alternate_beam
         self.__f = Param("f", self, frequency)
     
         self._freeze()
     
+    @property
+    def n(self): return self.__mode[0]
+    @n.setter
+    def n(self, value):
+        self.__mode[0] = int(value)
+        
+    @property
+    def m(self): return self.__mode[1] 
+    @m.setter
+    def m(self, value):
+        self.__mode[1] = int(value)
+        
     @property
     def mode(self): return self.__mode
     @mode.setter
@@ -425,9 +438,9 @@ class ad(Detector1):
             alt = ''
         
         if self.mode is None:
-            rtn.append("ad {name} {f} {node}{alt}".format(name=self.name, f=str(self.f.value), node=self.node.name, alt=alt))
+            rtn.append("ad {name} {f} {node}{alt}".format(name=self.name, f=self.f, node=self.node.name, alt=alt))
         else:
-            rtn.append("ad {name} {n} {m} {f} {node}{alt}".format(name=self.name, n=str(self.mode[0]), m=str(self.mode[1]), f=str(self.f.value), node=self.node.name, alt=alt))
+            rtn.append("ad {name} {n} {m} {f} {node}{alt}".format(name=self.name, n=str(self.mode[0]), m=str(self.mode[1]), f=self.f, node=self.node.name, alt=alt))
             
         for p in self._params:
             rtn.extend(p.getFinesseText())
@@ -572,6 +585,46 @@ class pd(Detector1):
         
         return result
 
+    def _get_fphi_str(self):
+        """
+        Will return the frequency phase pairs in a string for use
+        in making the finesse commands for various detectors
+        """
+        fphi_str = ""
+            
+        for n in range(1, 1+self.num_demods):
+            _f = self.__getattribute__("f"+str(n))
+            
+            if _f == "$fs":
+                fphi_str += " $fs"
+            else:
+                fphi_str += " {0}".format(_f)
+                
+            phi_val = self.__getattribute__("phase"+str(n))
+            
+            if phi_val != None:
+                if type(phi_val) == float:
+                    fphi_str += " {}".format(phi_val)
+                else:
+                    fphi_str += " {0}".format(phi_val)
+        
+        return fphi_str
+    
+    @staticmethod
+    def _parse_fphi(f, phs):
+        dict = {}
+        
+        for i in range(len(f)):
+            dict['f{0}'.format(i+1)] = SIfloat(f[i])
+            
+        for i in range(len(phs)):
+            if phs[i] == "max":
+                dict['phase{0}'.format(i+1)] = "max"
+            else:
+                dict['phase{0}'.format(i+1)] = SIfloat(phs[i])
+        
+        return dict
+    
     def __init__(self, name=None, num_demods=1, node_name=None, senstype=None, alternate_beam=False, pdtype=None, **kwargs):
         BaseDetector.__init__(self, name, node_name)
         
@@ -729,16 +782,8 @@ class pd(Detector1):
         
         f = values[2:len(values)-1:2]    
         phs = values[3:len(values)-1:2]
-        
-        dict = {}
-        
-        for i in range(len(f)):
-            dict['f{0}'.format(i+1)] = SIfloat(f[i])
-        for i in range(len(phs)):
-            if phs[i] == "max":
-                dict['phase{0}'.format(i+1)] = "max"
-            else:
-                dict['phase{0}'.format(i+1)] = SIfloat(phs[i])
+                
+        dict = pd._parse_fphi(f, phs)
             
         node = values[-1]
         alt_beam = node[-1] == '*'
@@ -754,26 +799,11 @@ class pd(Detector1):
         
         if self.enabled:
             alt_str = ""
-            fphi_str = ""
             
             if self.alternate_beam:
                 alt_str = "*"
-                
-            for n in range(1, 1+self.num_demods):
-                _f = self.__getattribute__("f"+str(n))
-                
-                if _f == "$fs":
-                    fphi_str += " $fs"
-                else:
-                    fphi_str += " {0:.16g}".format(float(_f))
-                    
-                phi_val = self.__getattribute__("phase"+str(n))
-                
-                if phi_val != None:
-                    if type(phi_val) == float:
-                        fphi_str += " {0:.16g}".format(float(phi_val))
-                    else:
-                        fphi_str += " {0}".format(phi_val)
+            
+            fphi_str = self._get_fphi_str()
             
             senstype = self.senstype
             
@@ -789,7 +819,7 @@ class pd(Detector1):
                 
             for p in self._params:
                 rtn.extend(p.getFinesseText())
-            
+        
         return rtn
   
 class qnoised(pd):
@@ -843,12 +873,7 @@ class qnoised(pd):
         f = values[3:len(values)-1:2]    
         phs = values[4:len(values)-1:2]
         
-        dict = {}
-        
-        for i in range(len(f)):
-            dict['f{0}'.format(i+1)] = f[i]
-        for i in range(len(phs)):
-            dict['phase{0}'.format(i+1)] = phs[i]
+        dict = pd._parse_fphi(f, phs)
             
         node = values[-1]
         alt_beam = node[-1] == '*'
@@ -870,27 +895,11 @@ class qnoised(pd):
         
         if self.enabled:
             alt_str = ""
-            fphi_str = ""
+            fphi_str = fphi_str = self._get_fphi_str()
             
             if self.alternate_beam:
                 alt_str = "*"
             
-            for n in range(1, 1+self.num_demods):
-                _f = self.__getattribute__("f"+str(n))
-                
-                if _f == "$fs":
-                    fphi_str += " $fs"
-                else:
-                    fphi_str += " {0:.16g}".format(float(_f))
-                    
-                phi_val = self.__getattribute__("phase"+str(n))
-                
-                if phi_val != None:
-                    if type(phi_val) == float:
-                        fphi_str += " {0:.16g}".format(float(phi_val))
-                    else:
-                        fphi_str += " " + str(phi_val)
-            
             senstype = self.senstype
             
             if senstype is None:
@@ -941,12 +950,7 @@ class qshot(pd):
         f = values[3:len(values)-1:2]    
         phs = values[4:len(values)-1:2]
         
-        dict = {}
-        
-        for i in range(len(f)):
-            dict['f{0}'.format(i+1)] = f[i]
-        for i in range(len(phs)):
-            dict['phase{0}'.format(i+1)] = phs[i]
+        dict = pd._parse_fphi(f, phs)
             
         node = values[-1]
         alt_beam = node[-1] == '*'
@@ -968,27 +972,11 @@ class qshot(pd):
         
         if self.enabled:
             alt_str = ""
-            fphi_str = ""
+            fphi_str = fphi_str = self._get_fphi_str()
             
             if self.alternate_beam:
                 alt_str = "*"
-                
-            for n in range(1, 1+self.num_demods):
-                _f = self.__getattribute__("f"+str(n))
-                
-                if _f == "$fs":
-                    fphi_str += " $fs"
-                else:
-                    fphi_str += " {0:.16g}".format(float(_f))
-                    
-                phi_val = self.__getattribute__("phase"+str(n))
-                
-                if phi_val != None:
-                    if type(phi_val) == float:
-                        fphi_str += " {0:.16g}".format(float(phi_val))
-                    else:
-                        fphi_str += " " + str(phi_val)
-            
+             
             senstype = self.senstype
             
             if senstype is None:
